'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getPundleInstance = exports.TICK = exports.X = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var getPundleInstance = exports.getPundleInstance = function () {
  var _ref = _asyncToGenerator(function* (cli, terminal, projectPath, development, givenConfig, errorCallback) {
    var config = normalizeConfig(projectPath, givenConfig);
    var pundleEntry = config.includePolyfills && Array.isArray(givenConfig.babel.presets) && givenConfig.babel.presets.indexOf('babel-preset-motion') !== -1 ? [require.resolve('babel-regenerator-runtime'), 'index.js'] : ['index.js'];
    var pundleConfig = {
      entry: pundleEntry,
      pathType: config.pathType === 'number' ? 'number' : 'filePath',
      rootDirectory: config.bundleDirectory,
      replaceVariables: {
        'process.env.NODE_ENV': development ? 'development' : 'production'
      }
    };

    var plugins = [[require.resolve('pundle-npm-installer'), {
      save: config.saveNpmModules,
      rootDirectory: config.bundleDirectory,
      beforeInstall: function beforeInstall(name) {
        if (terminal) {
          var message = 'Installing ' + name;
          cli.addSpinner(message);
        }
      },
      afterInstall: function afterInstall(name, error) {
        if (terminal) {
          var message = 'Installing ' + name;
          cli.removeSpinner(message);
          if (error) {
            cli.log('Install ' + name + ' ' + X);
            cli.log(error);
          } else {
            cli.log('Install ' + name + ' ' + TICK);
          }
        } else if (error) {
          errorCallback(error);
        }
      }
    }], [require.resolve('babel-pundle'), { config: config.babel }]];

    if (!development) {
      var _pundle = new _pundle3.default(pundleConfig);
      yield _pundle.loadPlugins(plugins);
      return _pundle;
    }
    var pundle = new _pundleDev2.default({
      server: {
        hmr: true,
        port: config.webServerPort,
        hmrPath: '/_/bundle_hmr',
        bundlePath: '/_/bundle.js',
        sourceRoot: config.publicDirectory,
        sourceMapPath: '/_/bundle.js.map',
        error: function error(_error) {
          errorCallback(_error);
        }
      },
      pundle: pundleConfig,
      watcher: {},
      generator: {
        wrapper: 'hmr',
        sourceMap: true
      }
    });
    yield pundle.pundle.loadPlugins(plugins);
    pundle.server.use('*', function serveRequest(req, res, next) {
      var error = arguments.length <= 3 || arguments[3] === undefined ? false : arguments[3];

      if (['/_/bundle.js', '/_/bundle.js.map', '/_/bundle_hmr'].indexOf(req.baseUrl) !== -1) {
        next();
        return;
      }
      (0, _send2.default)(req, req.baseUrl, { root: config.publicDirectory, index: 'index.html' }).on('error', function () {
        if (error) {
          next();
          return;
        }
        req.baseUrl = '/index.html';
        serveRequest(req, res, next, true);
      }).on('directory', function () {
        return next();
      }).pipe(res);
    });
    pundle.pundle.onDidProcess(function (_ref2) {
      var filePath = _ref2.filePath;

      if (filePath.indexOf('$root') === 0 && filePath.indexOf('node_modules') === -1 && filePath.indexOf('../') === -1) {
        cli.log(_chalk2.default.dim(filePath) + ' ' + _chalk2.default.green(TICK));
      }
    });
    return pundle;
  });

  return function getPundleInstance(_x, _x2, _x3, _x4, _x5, _x6) {
    return _ref.apply(this, arguments);
  };
}();

exports.getRandomNumber = getRandomNumber;
exports.normalizeConfig = normalizeConfig;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _send = require('send');

var _send2 = _interopRequireDefault(_send);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _pundle2 = require('pundle');

var _pundle3 = _interopRequireDefault(_pundle2);

var _pundleDev = require('pundle-dev');

var _pundleDev2 = _interopRequireDefault(_pundleDev);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

var X = exports.X = '✗';
var TICK = exports.TICK = '✓';

// From: goo.gl/fZA6BF
function getRandomNumber(min, max) {
  return Math.floor(Math.random() * (max - min)) + min;
}

// NOTE: The reason we are not replacing these in config is because when we then save the config
// We'd end up with absolute paths in it, using this function in every config user function
// will let us avoid that.
function normalizeConfig(projectPath, givenConfig) {
  var config = Object.assign({}, givenConfig, {
    babel: Object.assign({}, givenConfig.babel)
  });
  if (config.bundleDirectory.substr(0, 1) === '.') {
    config.bundleDirectory = _path2.default.resolve(projectPath, config.bundleDirectory);
  }
  if (config.publicDirectory.substr(0, 1) === '.') {
    config.publicDirectory = _path2.default.resolve(projectPath, config.publicDirectory);
  }
  if (!config.babel || _typeof(config.babel) !== 'object') {
    config.babel = { plugins: [], presets: [] };
  }
  if (!Array.isArray(config.babel.plugins)) {
    config.babel.plugins = [];
  } else {
    config.babel.plugins = config.babel.plugins.slice();
  }
  if (!Array.isArray(config.babel.presets)) {
    config.babel.presets = [];
  } else {
    config.babel.presets = config.babel.presets.slice();
  }
  if (config.babel.presets.indexOf('babel-preset-motion') !== -1) {
    config.babel.presets.splice(config.babel.presets.indexOf('babel-preset-motion'), 1, require.resolve(config.includePolyfills ? 'babel-preset-es2015' : 'babel-preset-es2015-sane'), require.resolve('babel-preset-motion'));
  }
  config.babel.plugins = config.babel.plugins.map(function (givenEntry) {
    if (!givenEntry) {
      return givenEntry;
    }
    var entry = Array.isArray(givenEntry) ? givenEntry : [givenEntry, {}];
    if (entry[0] === 'gloss/transform') {
      entry[0] = require.resolve('gloss/transform');
    }
    if (!_path2.default.isAbsolute(entry[0])) {
      if (entry[0].substr(0, 1) === '.') {
        entry[0] = _path2.default.resolve(projectPath, entry[0]);
      } else {
        entry[0] = _path2.default.join(projectPath, 'node_modules', entry[0]);
      }
    }
    return entry;
  });
  return config;
}
//# sourceMappingURL=helpers.js.map